#ifndef CvsInterface_h
#define CvsInterface_h

// $Id: CvsInterface.h,v 1.44 2012/11/22 22:25:31 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "Counter.h"
#include "CustomProcess.h"
#include "CvsCommand.h"
#include "FileInfo.h"
#include "MessageBuffer.h"
#include "Revision.h"
#include "TextFormat.h"

#include <QtGui/QColor>
#include <QtCore/QObject>
#include <QtCore/QString>

//! cvs log parser
/*!
commands are stored in a stack and processed asynchroneously
proper signal is emmited whenever a command is completed.
The stack is cleared when one command fails.
*/
class CvsInterface: public QObject, public Counter
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor
    CvsInterface( QObject* parent );

    //! run "cvs log" for a given file
    void logRevisions( const File& file );

    //! run "cvs status" for a given file/directory
    void logStatus( const File& path, const FileInfo::List& files = FileInfo::List() );

    //! fake update "cvs -n up".
    /*! Needed to get files remotely added and conflicts */
    void fakeUpdate( const File& path, const FileInfo::List& files = FileInfo::List() );

    //! check out a given revision for a given file
    void checkOutRevision( const File& source, const QString& revision );

    //! checkout
    void checkOut( const File& path, const QString& cvs_root, const QStringList& modules, const File& local_path );

    //! update
    void update( const File& path, const FileInfo::List& files = FileInfo::List() );

    //! add
    void add( const File& path, const FileInfo::List& files );

    //! remove
    void remove( const File& path, const FileInfo::List& files );

    //! commit
    void commit( const File& path, const File& comments, const FileInfo::List& file = FileInfo::List() );

    //! returns true if path is a valid CVS path
    /*! look for CVS/Root and CVS/Repository in the directory */
    bool isCvsPath( File path ) const;

    //! return true if file is in CVS/Entries
    bool isCvsFile( File file ) const;

    //! update repository and root from path
    /*! returns true on success */
    bool updateRepositoryPath( File path );

    //! update entries from path
    bool updateEntries( File path );

    //! returns current cvs root path
    QString root() const
    { return root_; }

    //! returns current repository path
    QString repository( void )
    { return repository_; }

    //! terminate process and clear stack
    void abort( int command_type = CvsCommand::ANY );

    //! verbosity
    enum Verbosity
    {
        LOW,
        HIGH
    };

    signals:

    //! emitted when a new scratch file is added
    void scratchFileAdded( const File& );

    //! emitted when revisions are available
    void revisionsAvailable( const File&, const Revision::List& );

    //! emitted when checkout is available
    /*! send the source file name and the destination filename (to which the revision is saved) */
    void revisionAvailable( const File&, const File& );

    //! emitted when status is available
    void statusAvailable( const File&, const FileInfo::List& );

    //! new command added
    void commandAdded( const CvsCommand& );

    //! command removed (prior to start)
    void commandRemoved( const CvsCommand& );

    //! new command started
    void commandStarted( const CvsCommand& );

    //! command completed
    void commandCompleted( const CvsCommand& );

    //! emitted for logging of processes
    void messageAvailable( const QString&, FORMAT::TextFormatFlags = FORMAT::Default, const QColor& = QColor(), int = 0 );

    public slots:

    //! terminate process and clear stack
    void abort( const CvsCommand& command );

    private slots:

    //! error from process
    void _error( QProcess::ProcessError );

    //! read on-fly from process
    void _readStdout( void );

    //! read on-fly from process
    void _readStderr( void );

    //! parse buffer
    void _parseStdoutBuffer( void );

    //! parse buffer
    void _parseStderrBuffer( void );

    //! cvs log command completed
    void _commandCompleted( int, QProcess::ExitStatus );

    protected:

    //! reset buffer
    void _resetBuffers( void );

    //! add commands to the pile
    void _addCommand( CvsCommand command );

    //! process next command in list
    void _run( void );

    //! run cvs status command
    void _runStatusCommand( void );

    //! run cvs status command
    void _runFakeUpdateCommand( void );

    //! run cvs checkout command
    void _runCheckOutCommand( void );

    //! run cvs update command
    void _runUpdateCommand( void );

    //! run cvs commit command
    void _runCommitCommand( void );

    //! check for lock messages
    bool _checkLock( const QString& buffer );

    //! process output from log command
    // void _logCommandCompleted( void );

    //! process output from log command
    void _logCommandAvailable( void );

    //! process output from status command
    void _statusCommandAvailable( void );

    //! process output from fake update command
    void _fakeUpdateCommandAvailable( void );

    //! process output from revision checkout command
    void _revisionCheckOutCommandCompleted( void );

    //! process output from checkout command
    void _checkOutCommandAvailable();

    //! process output from update command
    void _updateCommandAvailable();

    //! process output from cvs commit command
    void _commitCommandAvailable();

    //! process output from cvs commit command
    void _commitErrorAvailable();

    //! process output from cvs commit command
    void _commitCommandCompleted();

    //! reset process
    void _resetProcess( void );

    //! create new process to run command
    CustomProcess& _newProcess( void );

    //! process
    CustomProcess& _process( void ) const
    { return *process_; }

    private:

    //! local CVS Root directory
    QString root_;

    //! local CVS repository
    QString repository_;

    //! list of local CVS entries
    QList<File> entries_;

    //! process
    CustomProcess* process_;

    //! stdout buffer
    MessageBuffer stdoutBuffer_;

    //! stderr buffer
    MessageBuffer stderrBuffer_;

    //! if true locks are ignored, meaning they do not popup a dialog
    bool ignoreLocks_;

    //! pending commands
    QList<CvsCommand> commands_;

};

#endif
