// $Id: FileIconProvider.cpp,v 1.6 2012/12/18 11:40:52 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "FileIconProvider.h"
#include "FileInfo.h"

#include "CustomPixmap.h"
#include "Icons.h"
#include "IconEngine.h"

//__________________________________________________________________
FileIconProvider::FileIconProvider( QObject* parent ):
    BaseFileIconProvider( parent )
{

    // status icons, for all documents
    statusNames_[FileInfo::Unknown] = ICONS::UNKNOWN;
    statusNames_[FileInfo::UpToDate] = ICONS::UP_TO_DATE;
    statusNames_[FileInfo::LocallyModified] = ICONS::LOCALLY_MODIFIED;
    statusNames_[FileInfo::LocallyAdded] = ICONS::LOCALLY_ADDED;
    statusNames_[FileInfo::LocallyRemoved] = ICONS::LOCALLY_REMOVED;
    statusNames_[FileInfo::RemotelyModified] = ICONS::REMOTELY_MODIFIED;
    statusNames_[FileInfo::RemotelyAdded] = ICONS::REMOTELY_ADDED;
    statusNames_[FileInfo::RemotelyRemoved] = ICONS::REMOTELY_REMOVED;
    statusNames_[FileInfo::Merge] = ICONS::MERGE;
    statusNames_[FileInfo::Conflict] = ICONS::CONFLICT;

}

//__________________________________________________________________
const QIcon& FileIconProvider::icon( const BaseFileInfo& fileInfo )
{

    // try from base class
    const QIcon& icon( BaseFileIconProvider::icon( fileInfo ) );
    if( !icon.isNull() ) return icon;

    // get relevant file info type
    int type( fileInfo.type() );
    int status( type >> FileInfo::Shift );

    type &= (FileInfo::Document|FileInfo::Folder|FileInfo::Link|FileInfo::Hidden|FileInfo::Navigator);
    type |= status<<FileInfo::Shift;

    // build key
    Key key( fileInfo.file().extension(), type );

    // check if already created
    IconCache::const_iterator iter( icons_.find( key ) );
    if( iter != icons_.end() ) return iter.value();

    // create
    QIcon base;
    if( type & FileInfo::Navigator ) base = IconEngine::get( ICONS::PARENT );
    else if( type & FileInfo::Folder ) base = IconEngine::get( ICONS::FOLDER );
    else if( type & FileInfo::Document )
    {

        // try get icon from mimetypes
        base = mimeTypeIconProvider_.icon( fileInfo.file().extension() );
        if( base.isNull() ) base = IconEngine::get( ICONS::DOCUMENT );

    }

    // add link overlay
    if( type & FileInfo::Link )
    {

        QIcon copy( base );
        base = QIcon();
        QIcon linkOverlay( IconEngine::get( ICONS::LINK ) );
        foreach( const QSize& size, copy.availableSizes() )
        {

            QSize overlaySize;
            if( size.width() <= 16 ) overlaySize = QSize( 8, 8 );
            else if( size.width() <= 22 ) overlaySize = QSize( 8, 8 );
            else if( size.width() <= 32 ) overlaySize = QSize( 12, 12 );
            else if( size.width() <= 48 ) overlaySize = QSize( 16, 16 );
            else if( size.width() <= 64 ) overlaySize = QSize( 22, 22 );
            else if( size.width() <= 128 ) overlaySize = QSize( 48, 48 );
            else overlaySize = QSize( 64, 64 );

            base.addPixmap( CustomPixmap( copy.pixmap( size ) ).merge( linkOverlay.pixmap( overlaySize ).scaled( overlaySize, Qt::IgnoreAspectRatio, Qt::SmoothTransformation ), CustomPixmap::BOTTOM_RIGHT ) );

        }
    }

    // add status
    if( (type & FileInfo::Document) && statusNames_.contains( status ) )
    {

        QIcon copy( base );
        base = QIcon();
        const CustomPixmap statusOverlay( CustomPixmap().find( statusNames_[status] ) );
        foreach( const QSize& size, copy.availableSizes() )
        {
            base.addPixmap(
                CustomPixmap( copy.pixmap( size ) ).merge(
                statusOverlay.scaled( 0.4*size, Qt::KeepAspectRatio, Qt::SmoothTransformation ) ) );
        }

    }

    // hidden file
    if( type & FileInfo::Hidden )
    {
        QIcon copy( base );
        base = QIcon();
        foreach( const QSize& size, copy.availableSizes() )
        { base.addPixmap( CustomPixmap( copy.pixmap( size ) ).transparent( 0.6 ) ); }

    }

    // insert in map and return
    return icons_.insert( key, base ).value();

}
