
// $Id: FileInfoCache.cpp,v 1.22 2012/11/22 22:25:31 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "Debug.h"
#include "FileInfoCache.h"
#include "XmlDef.h"

#include <algorithm>
#include <QtXml/QDomDocument>
#include <QtXml/QDomElement>

//____________________________________________________________________________
FileInfoCache::FileInfoCache( const QDomElement& element ):
Counter( "FileInfoCache" ),
maxSize_( -1 )
{

    Debug::Throw( "FileInfoCache::FileInfoCache (dom).\n" );

    // parse children elements
    for(QDomNode child_node = element.firstChild(); !child_node.isNull(); child_node = child_node.nextSibling() )
    {
        QDomElement child_element = child_node.toElement();
        if( child_element.tagName() == XML::FILEINFO ) append( FileInfo( child_element ) );
        else Debug::Throw(0) << "FileInfoCache::FileInfoCache - unrecognized child " << child_element.tagName() << ".\n";
    }

    return;
}

//____________________________________________________________________________
QDomElement FileInfoCache::domElement( QDomDocument& parent ) const
{

    Debug::Throw( "FileInfoCache::domElement (dom).\n" );
    QDomElement out( parent.createElement( XML::FILEINFO_CACHE ) );
    for( const_iterator iter = begin(); iter != end(); ++iter )
    { out.appendChild( iter->domElement( parent ) ); }
    return out;
}

//____________________________________________________________________________
FileInfoCache FileInfoCache::findAll( File path ) const
{
    Debug::Throw( "FileInfoCache::findAll.\n" );

    Q_ASSERT( path.isDirectory() );

    FileInfoCache out;
    for( const_iterator iter = begin(); iter != end(); ++iter )
    {
        if( iter->file().path().isEqual( path ) )
        { out.insert( *iter ); }
    }

    return out;

}

//____________________________________________________________________________
void FileInfoCache::merge( const FileInfoCache& cache )
{
    Debug::Throw( "FileInfoCache::merge.\n" );
    for( const_iterator iter = cache.begin(); iter != cache.end(); ++iter )
    { insert( *iter ); }
    return;
}

//____________________________________________________________________________
bool FileInfoCache::insert( const FileInfo& file )
{

    iterator iter( std::find( begin(), end(), file ) );
    if( iter == end() )
    {
        append( file );
        return true;
    } else if( iter->created() < file.created() ) {
        *iter=  file;
        return true;
    } else {
        return false;
    }

}

//____________________________________________________________________________
FileInfoCache& FileInfoCache::truncate( void )
{
    Debug::Throw( "FileInfoCache::truncate.\n" );
    if( maxSize_ < 0 ) return *this;
    if( maxSize_ == 0 ) { clear(); return *this; }
    if( int(size()) <= maxSize_ ) return *this;

    // sort list
    std::sort( begin(), end(), FileInfo::FirstCreatedFTor() );

    // truncate to max_size
    // removing the first created info first
    while( int( size() ) > maxSize_ ) removeFirst();
    return *this;
}
