// $Id: MainWindow.cpp,v 1.64 2013/02/04 11:38:42 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "MainWindow.h"
#include "AnimatedTreeView.h"
#include "Application.h"
#include "BaseStatusBar.h"
#include "ClockLabel.h"
#include "Command.h"
#include "AnimatedTextEditor.h"
#include "FileDialog.h"
#include "CustomToolBar.h"
#include "CvsCommandDialog.h"
#include "FileInfoCacheDialog.h"
#include "FileInformationDialog.h"
#include "FileList.h"
#include "FileSelectionFrame.h"
#include "Icons.h"
#include "IconEngine.h"
#include "InformationDialog.h"
#include "LogDialog.h"
#include "LogWidget.h"
#include "Menu.h"
#include "PathEditor.h"
#include "QtUtil.h"
#include "RecentFilesMenu.h"
#include "RevisionCacheDialog.h"
#include "RevisionFrame.h"
#include "ScratchFileMonitor.h"
#include "Singleton.h"
#include "Util.h"
#include "ViewModeMenu.h"
#include "XmlOptions.h"

#include <QtGui/QApplication>
#include <QtGui/QLabel>
#include <QtGui/QGroupBox>
#include <QtGui/QSplitter>

//___________________________________________________________
MainWindow::MainWindow( QWidget* parent ):
    BaseMainWindow( parent ),
    Counter( "MainWindow" )
{

    setOptionName( "MAIN_WINDOW" );

    Debug::Throw( "MainWindow::MainWindow.\n" );
    _installActions();
    Application& application( *Singleton::get().application<Application>() );
    addAction( &application.closeAction() );

    // main widget and layout
    QWidget* main = new QWidget( this );
    QVBoxLayout* mainLayout( new QVBoxLayout() );
    main->setLayout( mainLayout );
    mainLayout->setMargin(0);
    mainLayout->setSpacing(0);
    setCentralWidget( main );

    // splitter
    QSplitter *splitter = new QSplitter( main );
    mainLayout->addWidget( splitter, 1 );
    splitter->setOrientation( Qt::Horizontal );

    //! dialogs
    commandDialog_ = new CvsCommandDialog();
    logDialog_ = new LogDialog();

    connect( commandDialog_, SIGNAL( dialogHidden() ), SLOT( _commandDialogHidden() ) );
    connect( logDialog_, SIGNAL( dialogHidden() ), SLOT( _logDialogHidden() ) );

    // file selection frame on the left
    fileSelectionFrame_ = new FileSelectionFrame( splitter );
    fileSelectionFrame_->editor().setParent( main );
    mainLayout->insertWidget( 0, &fileSelectionFrame_->editor() );

    // second splitter on the right
    splitter = new QSplitter( splitter );
    splitter->setOrientation( Qt::Vertical );
    revisionFrame_ = new RevisionFrame( splitter );

    if( XmlOptions::get().contains( "REVISION_FRAME_HEIGHT" ) )
    { revisionFrame_->setDefaultHeight( XmlOptions::get().get<int>( "REVISION_FRAME_HEIGHT" ) ); }

    // right vbox for buttons and comments
    QWidget *right = new QWidget( splitter );
    QVBoxLayout* vLayout = new QVBoxLayout();
    right->setLayout( vLayout );
    vLayout->setSpacing(0);
    vLayout->setMargin(0);

    // comments
    QLabel* label;
    vLayout->addWidget( label = new QLabel( "Comments:", right ) );
    vLayout->addWidget( commentsEditor_ = new AnimatedTextEditor( right ), 1 );

    label->setMargin( 5 );
    label->setBuddy( commentsEditor_ );

    commentsEditor_->setReadOnly( true );
    commentsEditor_->setWrapFromOptions( false );
    commentsEditor_->wrapModeAction().setChecked( true );
    commentsEditor_->setFrameStyle( QFrame::NoFrame );

    commentsEditor_->setForegroundRole( QPalette::WindowText );
    commentsEditor_->setBackgroundRole( QPalette::Window );

    commentsEditor_->viewport()->setForegroundRole( QPalette::WindowText );
    commentsEditor_->viewport()->setBackgroundRole( QPalette::Window );
    commentsEditor_->blockHighlight().setEnabled( false );

    // menu bar
    setMenuBar( menu_ = new Menu( this ) );
    connect( &menu_->viewModeMenu(), SIGNAL( viewModeSelected( FileSelectionFrame::ViewMode ) ), fileSelectionFrame_, SLOT( setViewMode( FileSelectionFrame::ViewMode ) ) );

    // status bar
    setStatusBar( statusbar_ = new BaseStatusBar( this ) );
    statusbar_->addLabels(1, 1 );
    statusbar_->addClock();

    // navigation toolbar
    CustomToolBar* toolbar;
    toolbar = new CustomToolBar( "Navigation", this, "NAVIGATION_TOOLBAR" );
    toolbar->addAction( &fileSelectionFrame_->homeDirectoryAction() );
    toolbar->addAction( &fileSelectionFrame_->previousDirectoryAction() );
    toolbar->addAction( &fileSelectionFrame_->nextDirectoryAction() );
    toolbar->addAction( &fileSelectionFrame_->parentDirectoryAction() );
    toolbar->addAction( &reloadAction() );

    toolbar->addSeparator();
    toolbar->addAction( &fileSelectionFrame_->cvsUpdateDirectoryAction() );
    toolbar->addAction( &fileSelectionFrame_->cvsCheckOutAction() );

    toolbar->addSeparator();
    toolbar->addAction( &fileSelectionFrame_->cvsAddAction() );
    toolbar->addAction( &fileSelectionFrame_->cvsRemoveAction() );
    toolbar->addAction( &fileSelectionFrame_->cvsCommitDirectoryAction() );

    toolbar->addSeparator();
    toolbar->addAction( &fileSelectionFrame_->filePropertiesAction() );

    // connections
    // file editor
    connect( &fileSelectionFrame_->editor(), SIGNAL( pathChanged( const File& ) ), SLOT( _reload( void ) ) );

    connect( fileSelectionFrame_, SIGNAL( fileSelected( File ) ), SLOT( _setFile( File ) ) );
    connect( fileSelectionFrame_, SIGNAL( requestFileInformation( const FileInfo&, const QIcon& ) ), SLOT( _fileInformation( const FileInfo&, const QIcon& ) ) );
    connect( fileSelectionFrame_, SIGNAL( messageAvailable( const QString&, FORMAT::TextFormatFlags, const QColor&, int ) ), &logDialog().editor(), SLOT( append( const QString&, FORMAT::TextFormatFlags, const QColor&, int ) ) );

    connect( &fileSelectionFrame_->interface(), SIGNAL( commandStarted( const CvsCommand& ) ), Singleton::get().application(), SLOT( busy() ) );
    connect( &fileSelectionFrame_->interface(), SIGNAL( commandCompleted( const CvsCommand& ) ), Singleton::get().application(), SLOT( idle() ) );
    connect( &fileSelectionFrame_->interface(), SIGNAL( commandAdded( const CvsCommand& ) ), &commandDialog().model(), SLOT( add( const CvsCommand& ) ) );
    connect( &fileSelectionFrame_->interface(), SIGNAL( commandRemoved( const CvsCommand& ) ), &commandDialog().model(), SLOT( remove( const CvsCommand& ) ) );
    connect( &fileSelectionFrame_->interface(), SIGNAL( commandCompleted( const CvsCommand& ) ), &commandDialog().model(), SLOT( remove( const CvsCommand& ) ) );
    connect( &fileSelectionFrame_->interface(), SIGNAL( commandCompleted( const CvsCommand& ) ), revisionFrame_, SLOT( commandCompleted( const CvsCommand& ) ) );
    connect( &fileSelectionFrame_->interface(), SIGNAL( messageAvailable( const QString&, FORMAT::TextFormatFlags, const QColor&, int ) ), &logDialog().editor(), SLOT( append( const QString&, FORMAT::TextFormatFlags, const QColor&, int ) ) );
    connect( &fileSelectionFrame_->interface(), SIGNAL( commandStarted( const CvsCommand& ) ), SLOT( _updateStatusBar( const CvsCommand& ) ) );
    connect( &fileSelectionFrame_->interface(), SIGNAL( commandCompleted( const CvsCommand& ) ), &statusbar_->label(), SLOT( clear() ) );

    connect( revisionFrame_, SIGNAL( currentRevisionChanged( const Revision& ) ), SLOT( _displayComments( const Revision& ) ) );
    connect( &revisionFrame_->interface(), SIGNAL( messageAvailable( const QString&, FORMAT::TextFormatFlags, const QColor&, int ) ), &logDialog().editor(), SLOT( append( const QString&, FORMAT::TextFormatFlags, const QColor&, int ) ) );
    connect( &revisionFrame_->interface(), SIGNAL( commandStarted( const CvsCommand& ) ), Singleton::get().application(), SLOT( busy() ) );
    connect( &revisionFrame_->interface(), SIGNAL( commandCompleted( const CvsCommand& ) ), Singleton::get().application(), SLOT( idle() ) );
    connect( &revisionFrame_->interface(), SIGNAL( commandAdded( const CvsCommand& ) ), &commandDialog().model(), SLOT( add( const CvsCommand& ) ) );
    connect( &revisionFrame_->interface(), SIGNAL( commandRemoved( const CvsCommand& ) ), &commandDialog().model(), SLOT( remove( const CvsCommand& ) ) );
    connect( &revisionFrame_->interface(), SIGNAL( commandCompleted( const CvsCommand& ) ), &commandDialog().model(), SLOT( remove( const CvsCommand& ) ) );

    // propagate command completed from revision frame to FileSelectionFrame
    // this to allow diff actions and checkout revision actions to toggle update of the list
    connect( &revisionFrame_->interface(), SIGNAL( commandCompleted( const CvsCommand& ) ), fileSelectionFrame_, SLOT( commandCompleted( const CvsCommand& ) ) );
    connect( &revisionFrame_->interface(), SIGNAL( commandStarted( const CvsCommand& ) ), SLOT( _updateStatusBar( const CvsCommand& ) ) );
    connect( &revisionFrame_->interface(), SIGNAL( commandCompleted( const CvsCommand& ) ), &statusbar_->label(), SLOT( clear() ) );
    connect( &revisionFrame_->interface(), SIGNAL( scratchFileAdded( const File& ) ), &Singleton::get().application<Application>()->scratchFileMonitor(), SLOT( add( const File& ) ) );

    connect( &application, SIGNAL( configurationChanged() ), SLOT( _updateConfiguration() ) );
    connect( qApp, SIGNAL( aboutToQuit() ), SLOT( _saveConfiguration() ) );
    _updateConfiguration();

    // read cache and assign to children
    cache_.read( QString( XmlOptions::get().raw( "CACHE_FILE" ) ) );
    fileSelectionFrame_->setCache( cache_.fileInfoCache() );
    revisionFrame_->setCache( cache_.revisionCache() );

    // splitter
    splitter->setStretchFactor( 0, 0 );
    splitter->setStretchFactor( 1, 1 );
    connect( splitter, SIGNAL( splitterMoved( int, int ) ), SLOT( _splitterMoved( void ) ) );

    // busy widget
    busyWidget_ = new BusyWidget( main, BusyWidget::BottomRight );
    busyWidget_->hide();

}

//__________________________________________________________________________
void MainWindow::open( const FileRecord& constRecord )
{

    Debug::Throw( "MainWindow::open.\n" );

    FileRecord record( constRecord );
    if( record.file().isEmpty() )
    {
        QString file( FileDialog( this ).selectFile( fileSelectionFrame_->path() ).getFile() );
        if( file.isNull() ) return;
        else record = FileRecord( file );
    }

    // update file selection frame
    fileSelectionFrame_->setFile( record.file() );

    // update fileEditor and menu
    _setFile( record.file() );

}

//_______________________________________________________
void MainWindow::clearCache( void )
{
    cache_.clear();
    fileSelectionFrame_->clearCache();
    revisionFrame_->clearCache();
}

//_______________________________________________________
void MainWindow::timerEvent( QTimerEvent* event )
{

    if( event->timerId() == resizeTimer_.timerId() )
    {

        resizeTimer_.stop();
        XmlOptions::get().set<int>( "REVISION_FRAME_HEIGHT", revisionFrame_->height() );

    } else return BaseMainWindow::timerEvent( event );

}

//____________________________________
void MainWindow::closeEvent( QCloseEvent *event )
{
    Debug::Throw( "MainWindow::closeEvent.\n" );
    _saveConfiguration();
    event->accept();
    qApp->quit();
}

//_______________________________________
void MainWindow::_updateConfiguration( void )
{

    Debug::Throw( "MainWindow::_updateConfiguration" );

    // command dialog
    if( XmlOptions::get().get<bool>( "SHOW_COMMAND_LIST" ) ) _showCommandDialog();
    else commandDialog().hide();

    // log dialog
    if( XmlOptions::get().get<bool>( "SHOW_LOG" ) ) _showLogDialog();
    else logDialog().hide();

    // reset comments editor
    commentsEditor_->blockHighlight().setEnabled( false );

}

///_______________________________________
void MainWindow::_saveConfiguration( void )
{

    Debug::Throw( "MainWindow::_saveConfiguration.\n" );

    // retrieve cache from children and write to file
    const qreal fileInfoScale( 1024.0*1024.0/182 );
    const qreal revisionScale( 1024.0*1024.0/153 );

    const int maxFileInfoCacheSize( fileInfoScale*XmlOptions::get().get<int>("FILEINFO_CACHE_SIZE") );
    const int maxRevisionCacheSize( revisionScale*XmlOptions::get().get<int>("REVISION_CACHE_SIZE") );
    cache_.setFileInfoCache( fileSelectionFrame_->cache().setMaximumSize( XmlOptions::get().get<bool>( "LIMIT_FILEINFO_CACHE" ) ? maxFileInfoCacheSize : -1 ).truncate() );
    cache_.setRevisionCache( revisionFrame_->cache().setMaximumSize( XmlOptions::get().get<bool>( "LIMIT_REVISION_CACHE" ) ? maxRevisionCacheSize:-1 ).truncate() );
    cache_.write();

}

//__________________________________________________________________________
void MainWindow::_reload( void )
{

    Debug::Throw( "MainWindow::_reload.\n" );

    // reload information based on editor contents
    QFileInfo file( fileSelectionFrame_->editor().path() );
    QFileInfo path( file.absolutePath() );
    if( path.exists() )
    {

        Debug::Throw() << "MainWindow::_reload - file: " << path.absoluteFilePath() << endl;
        fileSelectionFrame_->setFile( file.absoluteFilePath() );

    } else {

        // invalid path. Prompt dialog
        QString buffer;
        QTextStream( &buffer ) << "Path " << path.absoluteFilePath() << " is invalid. <Reload> canceled.";
        InformationDialog( this, buffer ).exec();

    }

}

//__________________________________________________________________________
void MainWindow::_setFile( const File& file )
{

    Debug::Throw() << "MainWindow::_setFile - file: " << file << endl;

    // update window title
    {
        File path( file );
        if( !path.isDirectory() ) path = path.path();
        if( path.endsWith('/') ) path = path.left( path.size()-1 );
        QString buffer;
        QTextStream what( &buffer );
        if( !path.isEmpty() ) what << path.localName() << " - ";
        what << "Cvswatch";
        setWindowTitle( buffer );
    }

    // clear comments if file differ from current revision file
    if( file != revisionFrame_->file() )
    {
        revisionFrame_->setFile( file.isDirectory() ? File():file );
        commentsEditor_->QTextEdit::clear();
    }

    // add to openPrevious menu
    Singleton::get().application<Application>()->recentFiles().add( file.isDirectory() ? file:file.path() );
    menu().recentFilesMenu().setCurrentFile( file.isDirectory() ? file:file.path() );

    return;
}

//_______________________________________________________________________
void MainWindow::_updateStatusBar( const CvsCommand& command )
{
    Debug::Throw( "MainWindow::_updateStatusBar.\n" );

    // create stream, add name, and possible argument
    QString buffer;
    QTextStream what( &buffer );
    what << command.name();

    if( command.argumentCount() )
    { what << " " << command.argument(0); }

    // update status bar
    statusbar_->label(0).setText( buffer );

}
//__________________________________________________________________________
void MainWindow::_showCommandDialog( void )
{

    Debug::Throw( "MainWindow::_showCommandDialog.\n" );
    XmlOptions::get().set<bool>( "SHOW_COMMAND_LIST", true );
    commandDialog().show();
    commandDialog().raise();

}

//_______________________________________________
void MainWindow::_commandDialogHidden( void )
{
    Debug::Throw( "Application::_commandDialogHidden.\n" );
    XmlOptions::get().set<bool>( "SHOW_COMMAND_LIST", false );
}

//__________________________________________________________________________
void MainWindow::_showLogDialog( void )
{

    Debug::Throw( "MainWindow::_showLogDialog.\n" );
    XmlOptions::get().set<bool>( "SHOW_LOG", true );
    logDialog().show();
    logDialog().raise();

}

//_______________________________________________
void MainWindow::_logDialogHidden( void )
{
    Debug::Throw( "MainWindow::_logDialogHidden.\n" );
    XmlOptions::get().set<bool>( "SHOW_LOG", false );
}

//_______________________________________________________________________
void MainWindow::_showFileInfoCache( void )
{

    Debug::Throw( "MainWindow::_showFileInfoCache.\n" );
    FileInfoCacheDialog dialog( this );
    dialog.setWindowTitle( "File Information - Cvswatch" );
    dialog.setCache( cache_.fileInfoCache() );
    dialog.centerOnParent().exec();

}

//_______________________________________________________________________
void MainWindow::_showRevisionCache( void )
{

    Debug::Throw( "MainWindow::_showRevisionCache.\n" );
    RevisionCacheDialog dialog( this );
    dialog.setWindowTitle( "Revisions - Cvswatch" );
    dialog.setCache( cache_.revisionCache() );
    dialog.centerOnParent().exec();

}

//_______________________________________________
void MainWindow::_fileInformation( const FileInfo& fileInfo, const QIcon& icon )
{

    FileInformationDialog dialog( this, fileInfo );
    dialog.setIcon( icon );
    dialog.exec();

}

//________________________________________________________
void MainWindow::_splitterMoved( void )
{
    Debug::Throw( "MainWindow::_splitterMoved.\n" );
    resizeTimer_.start( 200, this );
}

//_______________________________________________________________________
void MainWindow::_installActions( void )
{
    Debug::Throw( "MainWindow::_installActions.\n" );

    addAction( reloadAction_ = new QAction( IconEngine::get( ICONS::RELOAD ), "Reload", this ) );
    reloadAction_->setShortcut( QKeySequence::Refresh );
    connect( reloadAction_, SIGNAL( triggered() ), SLOT( _reload() ) );

    addAction( openAction_ = new QAction( IconEngine::get( ICONS::OPEN ), "Open...", this ) );
    openAction_->setShortcut( QKeySequence::Open );
    openAction_->setToolTip( "Select file and directory" );
    connect( openAction_, SIGNAL( triggered() ), SLOT( open() ) );

    addAction( fileInfoCacheAction_ = new QAction( "File Cache", this ) );
    fileInfoCacheAction_->setToolTip( "Show file cache" );
    connect( fileInfoCacheAction_, SIGNAL( triggered() ), SLOT( _showFileInfoCache() ) );

    addAction( revisionCacheAction_ = new QAction( "Revision Cache", this ) );
    revisionCacheAction_->setToolTip( "Show revision cache" );
    connect( revisionCacheAction_, SIGNAL( triggered() ), SLOT( _showRevisionCache() ) );

    addAction( showCommandDialogAction_ = new QAction( IconEngine::get( ICONS::FIND ), "Show Command List", this ) );
    connect( showCommandDialogAction_, SIGNAL( triggered( void ) ), SLOT( _showCommandDialog( void ) ) );

    addAction( showLogDialogAction_ = new QAction( IconEngine::get( ICONS::FIND ), "Show Log", this ) );
    connect( showLogDialogAction_, SIGNAL( triggered( void ) ), SLOT( _showLogDialog( void ) ) );

}
