

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "RevisionFrame.h"

#include "AnimatedTreeView.h"
#include "Command.h"
#include "ContextMenu.h"
#include "CvsInterface.h"
#include "Icons.h"
#include "IconEngine.h"
#include "InformationDialog.h"
#include "XmlOptions.h"

#include <QtGui/QLayout>

//________________________________________________________________
RevisionFrame::RevisionFrame( QWidget* parent ):
    QWidget( parent ),
    Counter( "RevisionFrame" ),
    defaultHeight_( -1 ),
    interface_( new CvsInterface( this ) )
{

    Debug::Throw( "RevisionFrame::RevisionFrame.\n" );
    QVBoxLayout* layout = new QVBoxLayout();
    layout->setMargin(0);
    layout->setSpacing(5);
    setLayout( layout );

    _installActions();

    //! list
    layout->addWidget( list_ = new AnimatedTreeView( this ), 1 );
    list_->setModel( &model_ );
    list_->setSortingEnabled( true );
    list_->setOptionName( "REVISION_LIST" );

    QMenu* menu( new ContextMenu( list_ ) );
    menu->addAction( &editLocalAction() );
    menu->addAction( &editRevisionAction() );
    menu->addAction( &diffAction() );

    // connections to CvsInterface
    connect( list_, SIGNAL( activated( const QModelIndex& ) ), SLOT( _editRevision() ) );
    connect( list_->selectionModel(), SIGNAL( selectionChanged(const QItemSelection &, const QItemSelection &) ), SLOT( _updateActions() ) );
    connect( list_->selectionModel(), SIGNAL( selectionChanged(const QItemSelection &, const QItemSelection &) ), SIGNAL( selectionChanged() ) );
    connect( list_->selectionModel(), SIGNAL( currentChanged(const QModelIndex&, const QModelIndex&) ), SLOT( _currentRevisionChanged( const QModelIndex& ) ) );
    connect( interface_, SIGNAL( revisionsAvailable( File, Revision::List ) ), SLOT( _showRevisions( File, Revision::List ) ) );
    connect( interface_, SIGNAL( commandCompleted( CvsCommand ) ), SLOT( commandCompleted( CvsCommand ) ) );
    _updateActions();

    return;
}

//________________________________________________________________
void RevisionFrame::clear( void )
{

    Debug::Throw( "RevisionFrame::clear.\n" );
    list_->initializeAnimation();
    model_.clear();
    file_.clear();

    list_->startAnimation();
}

//________________________________________________________________
void RevisionFrame::setFile( const File& file )
{
    Debug::Throw( "RevisionFrame::setFile.\n" );

    // see if file has changed
    if( file == file_ ) return;

    // reload entries
    if( !file.isEmpty() && (file_.isEmpty() || file.path() != file_.path() ) && interface_->isCvsPath( file.path() ) )
    { interface_->updateEntries( file.path() ); }

    file_ = file;
    if( file.isEmpty() )
    {
        clear();
        return;
    }

    // clear list
    list_->initializeAnimation();
    model_.clear();
    list_->startAnimation();

    // try load revisions from cache
    // then from CVS
    if( interface_->isCvsFile( file ) )
    {
        _showRevisionsFromLocalCache();

        // delayed CVS reload
        timer_.start( 500, this );
    }

}

//_______________________________________________________________________
void RevisionFrame::commandCompleted( const CvsCommand& command )
{

    Debug::Throw( "RevisionFrame::_commandCompleted.\n" );
    switch( command.type() )
    {
        case CvsCommand::COMMIT:
        _reload();
        break;

        case CvsCommand::LOG:
        if( !localCache_.empty() )
        {
            std::sort( localCache_.begin(), localCache_.end() );
            localCache_.erase( std::unique( localCache_.begin(), localCache_.end() ), localCache_.end() );
            cache_[file()] = localCache_;
        }
        break;

        default:
        break;
    }
}

//__________________________________________________________________________
void RevisionFrame::_updateActions( void )
{

    Debug::Throw( "RevisionFrame::_updateActions.\n" );

    bool file_exists( file().exists() );
    bool hasSelection( !list_->selectionModel()->selectedRows().empty() );

    editLocalAction().setEnabled( file_exists );
    editRevisionAction().setEnabled( hasSelection );
    diffAction().setEnabled( hasSelection && file_exists );

}

//__________________________________________________________________________
void RevisionFrame::timerEvent( QTimerEvent* event )
{
    if( event->timerId() == timer_.timerId() )
    {

        timer_.stop();
        _reload();

    } else return QWidget::timerEvent( event );

}

//__________________________________________________________________________
void RevisionFrame::_reload( void )
{

    Debug::Throw( "RevisionFrame::_reload.\n" );
    if( interface_->isCvsPath( file().path() ) ) interface_->updateEntries( file().path() );
    if( interface_->isCvsFile( file() ) )
    {
        interface_->abort();
        interface_->logRevisions( file() );
    }

}


//______________________________________________________________________
void RevisionFrame::_editLocal( void )
{

    Debug::Throw( "RevisionFrame::_editLocal.\n" );
    ( Command( XmlOptions::get().raw( "EDITOR" ) ) << file().expand() ).run();

}

//______________________________________________________________________
void RevisionFrame::_editRevision( void )
{

    Debug::Throw( "RevisionFrame::_editRevision.\n" );

    // check selection, retrieve revision
    QModelIndex current( list_->currentIndex() );
    if( !current.isValid() )
    {
        InformationDialog( this, "No revision selected. <edit> canceled." ).exec();
        return;
    }

    disconnect( interface_, SIGNAL( revisionAvailable( File, File ) ), this, SLOT( _diff( File, File ) ) );
    disconnect( interface_, SIGNAL( revisionAvailable( File, File ) ), this, SLOT( _editRevision( File, File ) ) );
    connect( interface_, SIGNAL( revisionAvailable( File, File ) ), this, SLOT( _editRevision( File, File ) ) );
    interface_->checkOutRevision( file(), model_.get( current ).name() );

}

//_______________________________________________________________________
void RevisionFrame::_diff( void )
{

    Debug::Throw( "RevisionFrame::_diff.\n" );

    // check selection, retrieve revision
    QModelIndex current( list_->currentIndex() );
    if( !current.isValid() )
    {
        InformationDialog( this, "No revision selected. <edit> canceled." ).exec();
        return;
    }

    disconnect( interface_, SIGNAL( revisionAvailable( File, File ) ), this, SLOT( _editRevision( File, File ) ) );
    disconnect( interface_, SIGNAL( revisionAvailable( File, File ) ), this, SLOT( _diff( File, File ) ) );
    connect( interface_, SIGNAL( revisionAvailable( File, File ) ), this, SLOT( _diff( File, File ) ) );
    interface_->checkOutRevision( file(), model_.get( current ).name() );

}

//__________________________________________________________________________
void RevisionFrame::_showRevisions( File file, Revision::List revisionList )
{
    Debug::Throw( "RevisionFrame::_showRevisions.\n" );

    // do nothing if file has changed
    if( file != file_ ) return;

    // add new items
    for( Revision::List::const_iterator iter = revisionList.begin(); iter != revisionList.end(); ++iter )
    { localCache_ << *iter; }
    model_.add( revisionList );

    if( !revisionList.empty() )
    {
        list_->resizeColumnToContents( RevisionModel::REVISION );
        list_->resizeColumnToContents( RevisionModel::AUTHOR );
        list_->resizeColumnToContents( RevisionModel::DATE );
    }

    // scroll to last item or selected
    if( list_->currentIndex().isValid() ) list_->scrollTo( list_->currentIndex() );
    else if( model_.rowCount() ) list_->scrollTo( model_.index( model_.rowCount()-1, 0 ) );

}

//______________________________________________________________________
void RevisionFrame::_editRevision( File, File destination )
{

    Debug::Throw() << "RevisionFrame::_editRevision - file: " << destination << endl;
    ( Command( XmlOptions::get().raw( "EDITOR" ) ) << destination.expand() ).run();
    return;

}

//_______________________________________________________________________
void RevisionFrame::_diff( File source, File destination )
{

    Debug::Throw() << "RevisionFrame::_diff - file: " << destination << endl;
    ( Command( XmlOptions::get().raw( "DIFF_EDITOR" ) ) << source.expand() << destination.expand() ).run();
    return;

}

//__________________________________________________________________________
void RevisionFrame::_installActions( void )
{

    Debug::Throw( "RevisionFrame::_installActions.\n" );

    // edit local file action
    addAction( editLocalAction_ = new QAction( IconEngine::get( ICONS::EDIT ), "&Edit Local File", this ) );
    connect( editLocalAction_, SIGNAL( triggered() ), SLOT( _editLocal() ) );

    // edit revision action
    addAction( editRevisionAction_ = new QAction( IconEngine::get( ICONS::EDIT ), "Edit &Revision", this ) );
    connect( editRevisionAction_, SIGNAL( triggered() ), SLOT( _editRevision() ) );

    // diff local and revision
    addAction( diffAction_ = new QAction( "&Diff Local and Revision", this ) );
    connect( diffAction_, SIGNAL( triggered() ), SLOT( _diff() ) );

}

//__________________________________________________________________________
void RevisionFrame::_showRevisionsFromLocalCache( void )
{
    Debug::Throw( "RevisionFrame::_showRevisionsFromLocalCache.\n" );

    // retrieve local_cache
    localCache_ = cache_[ file() ];
    if( localCache_.empty() ) return;

    // add new items
    for( Revision::List::const_iterator iter = localCache_.begin(); iter != localCache_.end(); ++iter )
    { model_.add( *iter ); }

    list_->resizeColumnToContents( RevisionModel::REVISION );
    list_->resizeColumnToContents( RevisionModel::AUTHOR );
    list_->resizeColumnToContents( RevisionModel::DATE );

    // scroll to last item or selected
    if( list_->currentIndex().isValid() ) list_->scrollTo( list_->currentIndex() );
    else if( model_.rowCount() ) list_->scrollTo( model_.index( model_.rowCount()-1, 0 ) );

}
